# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from copy import deepcopy
from datetime import datetime
from flask import current_app as app
from eve.io.mongo import Validator
from superdesk.metadata.item import ITEM_TYPE
from superdesk.logging import logger
from superdesk.text_utils import get_text
from superdesk import get_resource_service
from _collections_abc import MutableMapping
from superdesk.signals import item_validate
from superdesk.validator import BaseErrorHandler
from flask_babel import lazy_gettext, _


REQUIRED_FIELD = 'is a required field'
MAX_LENGTH = 'max length is {length}'
STRING_FIELD = 'require a string value'
DATE_FIELD = 'require a date value'
REQUIRED_ERROR = '{} is a required field'
INVALID_CHAR = 'contains invalid characters'
TOO_LONG = '{} is too long'
TOO_SHORT = '{} is too short'
EMPTY_VALUE = 'empty values not allowed'

FIELD_LABELS = {
    'abstract': lazy_gettext('Abstract'),
    'alt_text': lazy_gettext('Alt text'),
    'anpa_take_key': lazy_gettext('Take Key'),
    'archive_description': lazy_gettext('Archive description'),
    'attachments': lazy_gettext('Attachments'),
    'authors': lazy_gettext('Authors'),
    'body_footer': lazy_gettext('Body footer'),
    'body_html': lazy_gettext('Body HTML'),
    'byline': lazy_gettext('Byline'),
    'categories': lazy_gettext('Categories'),
    'company_codes': lazy_gettext('Company Codes'),
    'copyrightholder': lazy_gettext('Copyright holder'),
    'copyrightnotice': lazy_gettext('Copyright notice'),
    'dateline': lazy_gettext('Dateline'),
    'description_text': lazy_gettext('Description'),
    'desk': lazy_gettext('Desk'),
    'ednote': lazy_gettext('Ed. Note'),
    'embargo': lazy_gettext('Embargo'),
    'feature_image': lazy_gettext('Feature Image'),
    'feature_media': lazy_gettext('Feature Media'),
    'footer': lazy_gettext('Footer'),
    'footers': lazy_gettext('Footers'),
    'genre': lazy_gettext('Genre'),
    'headline': lazy_gettext('Headline'),
    'ingest_provider': lazy_gettext('Ingest Provider'),
    'keywords': lazy_gettext('Keywords'),
    'language': lazy_gettext('Language'),
    'media': lazy_gettext('Media'),
    'media_description': lazy_gettext('Media Description'),
    'place': lazy_gettext('Place'),
    'priority': lazy_gettext('Priority'),
    'publish_schedule': lazy_gettext('Scheduled Time'),
    'relatedItems': lazy_gettext('Related Items'),
    'sign_off': lazy_gettext('Sign Off'),
    'slugline': lazy_gettext('Slugline'),
    'sms': lazy_gettext('SMS'),
    'sms_message': lazy_gettext('SMS Message'),
    'source': lazy_gettext('Source'),
    'stage': lazy_gettext('Stage'),
    'subject': lazy_gettext('Subject'),
    'type': lazy_gettext('Type'),
    'urgency': lazy_gettext('Urgency'),
    'usageterms': lazy_gettext('Usage Terms'),
}

ERROR_MESSAGES = {
    REQUIRED_FIELD: lazy_gettext('is a required field'),
    MAX_LENGTH: lazy_gettext('max length is {length}'),
    STRING_FIELD: lazy_gettext('require a string value'),
    DATE_FIELD: lazy_gettext('require a date value'),
    REQUIRED_ERROR: lazy_gettext('{} is a required field'),
    INVALID_CHAR: lazy_gettext('contains invalid characters'),
    TOO_LONG: lazy_gettext('{} is too long'),
    TOO_SHORT: lazy_gettext('{} is too short'),
    EMPTY_VALUE: lazy_gettext('empty values not allowed'),
}


def check_json(doc, field, value):
    if isinstance(doc, dict):
        if field in doc and value['required'] and doc[field]:
            return True
        for key in doc:
            if check_json(doc[key], field, value):
                return True
        return False
    elif isinstance(doc, list):
        for item in doc:
            if check_json(item, field, value):
                return True
        return False
    else:
        return False


def get_validator_schema(schema):
    """Get schema for given data that will work with validator.

    - if field is required without minlength set make sure it's not empty
    - if there are keys with None value - remove them

    :param schema
    """
    validator_schema = {key: val for key, val in schema.items() if val is not None}

    if validator_schema.get('required') and not validator_schema.get('minlength'):
        validator_schema.setdefault('empty', False)

    return validator_schema


class SchemaValidator(Validator):

    def __init__(self, *args, **kwargs):
        kwargs['error_handler'] = BaseErrorHandler
        super(SchemaValidator, self).__init__(*args, **kwargs)

    types_mapping = {
        k: v for k, v in Validator.types_mapping.items()
        if k not in ('date', )
    }

    def _validate_type_media(self, value):
        return True

    def _validate_type_related_content(self, value):
        return True

    def _validate_type_embed(self, value):
        return True

    def _validate_type_custom(self, value):
        return True

    def _validate_type_picture(self, value):
        return True

    def _validate_type_any(self, value):
        """Allow type any, ex: for CV of type 'custom'."""
        return True

    def _validate_type_text(self, value):
        """Validate type text in schema."""
        if value and isinstance(value, str):
            return True

    def _validate_type_date(self, value):
        if isinstance(value, datetime):
            return True
        try:
            datetime.strptime(value or '', '%Y-%m-%dT%H:%M:%S+%f')
            return True
        except ValueError:
            pass

    def _validate_mandatory_in_list(self, mandatory, field, value):
        """
        {'type': 'dict'}
        """
        for key in mandatory:
            for key_field in mandatory[key]:
                if mandatory[key][key_field]["required"]:
                    if value and len(value) > 0 and any(value_field['scheme'] == key_field for value_field in value):
                        if not check_json(value, key, mandatory[key][key_field]):
                            self._error(key_field, REQUIRED_FIELD)
                    else:
                        self._error(key_field, REQUIRED_FIELD)

    def _validate_mandatory_in_dictionary(self, mandatory, field, value):
        """
        {'type': 'list'}
        """
        for key in mandatory:
            if not value.get(key):
                self._error(key, REQUIRED_FIELD)

    def _validate_empty(self, empty, field, value):
        """
        {'type': 'boolean'}
        """
        super()._validate_empty(empty, field, value)
        if field == "subject":
            # for subject, we have to ignore all data with scheme
            # as they are used for custom values except "subject_custom" scheme as it's the scheme for subject cv
            # so it must be present
            subject_schemas = {None, '', 'subject_custom'}

            # plus any cv with schema_field subject
            cvs = get_resource_service('vocabularies').get_from_mongo(
                req=None, lookup={'schema_field': field}, projection={'_id': 1})
            for cv in cvs:
                subject_schemas.add(cv['_id'])

            filtered = [v for v in value if v.get('scheme') in subject_schemas]

            if not filtered:
                self._error(field, REQUIRED_FIELD)

        elif isinstance(value, list) and not value:
            self._error(field, REQUIRED_FIELD)
        elif isinstance(value, str) and value == '<p></p>':  # default value for editor3
            self._error(field, REQUIRED_FIELD)

    def _validate_enabled(self, *args):
        """
        {'type': 'boolean'}
        """
        pass

    def _validate_readonly(self, *args):
        """
        {'type': 'boolean'}
        """
        pass

    def _validate_place(self, *args):
        """
        {'type': 'list'}
        """
        pass

    def _validate_genre(self, *args):
        """
        {'type': 'dict'}
        """
        pass

    def _validate_anpa_category(self, *args):
        """
        {'type': 'list'}
        """
        pass

    def _validate_subject(self, *args):
        """
        {'type': 'list'}
        """
        pass

    def _validate_company_codes(self, *args):
        """
        {'type': 'list'}
        """
        pass

    def _validate_validate_characters(self, validate, field, value):
        """
        {'type': 'boolean'}
        """
        disallowed_characters = app.config.get('DISALLOWED_CHARACTERS')

        if validate and disallowed_characters and value:
            invalid_chars = [char for char in disallowed_characters if char in value]
            if invalid_chars:
                return self._error(field, INVALID_CHAR)

    def _validate_media_metadata(self, validate, associations_field, associations):
        """
        {'type': 'boolean'}
        """
        if not validate:
            return
        media_metadata_schema = app.config.get('VALIDATOR_MEDIA_METADATA')
        if not media_metadata_schema:
            return
        for assoc_name, assoc_data in associations.items():
            if assoc_data is None or assoc_data.get('type') == 'text':
                continue
            for field, schema in media_metadata_schema.items():
                if schema.get('required', False) and not assoc_data.get(field):
                    self._error("media's " + field, REQUIRED_FIELD)
                try:
                    max_length = int(schema['maxlength'])
                except KeyError:
                    pass
                except ValueError:
                    logger.error('Invalid max length value for media field {field}'.format(field=field))
                else:
                    if assoc_data.get(field) is not None and len(assoc_data[field]) > max_length:
                        self._error("media's " + field, MAX_LENGTH.format(length=max_length))


class ValidateResource(superdesk.Resource):
    schema = {
        'act': {'type': 'string', 'required': True},
        'type': {'type': 'string', 'required': True},
        'embedded': {'type': 'boolean', 'required': False},
        'validate': {
            'type': 'dict',
            'required': True,
        },
        'errors': {'type': 'list'},
    }

    resource_methods = ['POST']
    item_methods = []


class ValidateService(superdesk.Service):

    def create(self, docs, fields=False, **kwargs):
        for doc in docs:
            test_doc = deepcopy(doc)
            doc['errors'] = self._validate(test_doc, fields=fields, **kwargs)
        if fields:
            return [doc['errors'] for doc in docs]
        return [i for i in range(len(docs))]

    def _get_profile_schema(self, schema, doc):
        doc['validate'].setdefault('extra', {})  # make sure extra is there so it will validate its fields
        extra_field_types = {'text': 'string', 'embed': 'dict', 'date': 'date',
                             'urls': 'list', 'custom': 'any'}
        extra_fields = superdesk.get_resource_service('vocabularies').get_extra_fields()
        schema['extra'] = {'type': 'dict', 'schema': {}}
        for extra_field in extra_fields:
            if schema.get(extra_field['_id']) and \
                    extra_field.get('field_type', None) in extra_field_types:
                rules = schema.pop(extra_field['_id'])
                rules['type'] = extra_field_types.get(extra_field['field_type'], 'string')
                schema['extra']['schema'].update({extra_field['_id']: get_validator_schema(rules)})
                self._populate_extra(doc['validate'], extra_field['_id'])
        try:
            # avoid errors when cv is removed and value is still there
            schema['subject']['schema']['schema']['scheme'].pop('allowed', None)
        except KeyError:
            pass
        return [{'schema': schema}]

    def _get_validators(self, doc):
        """Get validators.

        In case there is profile defined for item with respective content type it will
        use its schema for validations, otherwise it will fall back to action/item_type filter.
        """
        profile = doc['validate'].get('profile')
        item_type = doc['validate'].get('type') or doc.get('type')
        if (profile or item_type) and (app.config['AUTO_PUBLISH_CONTENT_PROFILE'] or doc['act'] != 'auto_publish'):
            content_type = superdesk.get_resource_service('content_types').find_one(req=None, _id=profile)
            if not content_type and item_type:
                content_type = superdesk.get_resource_service('content_types').find_one(req=None, item_type=item_type)
            if content_type:
                return self._get_profile_schema(content_type.get('schema', {}), doc)
        lookup = {'act': doc['act'], 'type': doc[ITEM_TYPE]}
        if doc.get('embedded'):
            lookup['embedded'] = doc['embedded']
        else:
            lookup['$or'] = [{'embedded': {'$exists': False}}, {'embedded': False}]
        custom_schema = app.config.get('SCHEMA', {}).get(doc[ITEM_TYPE])
        if custom_schema:  # handle custom schema like profile schema
            return self._get_profile_schema(custom_schema, doc)
        return superdesk.get_resource_service('validators').get(req=None, lookup=lookup)

    def _populate_extra(self, doc, schema):
        """Populates the extra field in the document with fields stored in subject. Used
        for user defined vocabularies.

        :param doc: Article to be validated
        :param schema: Vocabulary identifier
        """
        for subject in doc.get('subject', []):
            if subject.get('scheme', '') == schema:
                doc['extra'][schema] = subject.get('qcode', '')

    def _sanitize_fields(self, doc, validator):
        """If maxlength or minlength is specified in the validator then remove any markups from that field

        :param doc: Article to be validated
        :param validator: Validation rule
        :return: updated article
        """
        fields_to_check = ['minlength', 'maxlength']
        item_schema = validator.get('schema', {})
        extra_schema = item_schema.get('extra', {}).get('schema', {})
        schemes_docs = [(item_schema, doc), (extra_schema, doc.get('extra', {}))]
        for schema, content in schemes_docs:
            for field in schema:
                if content.get(field) and schema.get(field) and type(content[field]) is str and \
                        any(k in schema[field] for k in fields_to_check):
                    try:
                        content[field] = get_text(content[field])
                    except (ValueError, TypeError):
                        # fails for json fields like subject, genre
                        pass

    def _get_media_fields(self, field_schema, field_associations, doc,):
        """Returns the field names in associations. For multiple valued media fields
        the field asssociations name is different from the field schema name.
        :param field_schema: Field schema name
        :param field_associations: Field associations name
        :param doc: Article to be validated
        """

        fields = []
        if field_associations in doc.get('associations', {}):
            return [field_associations]
        media_multivalue_field = field_schema + '--'
        for media_field in doc.get('associations', {}):
            if media_field.startswith(media_multivalue_field) and \
                    doc.get('associations', {}).get(media_field):
                fields.append(media_field)
        return fields

    def _process_media(self, doc, validation_schema):
        """If media field(feature media or custom media field) is required it should be present for
        validation on doc not only on associations
        If media_description is required it should be present for validation on doc not only on
        associations->featuremedia (or custom media field)
        :param doc: Article to be validated
        :param schema: Schema to validate the article
        """

        for field_schema in validation_schema:
            field_associations = field_schema if field_schema != 'feature_media' else 'featuremedia'
            media_fields = self._get_media_fields(field_schema, field_associations, doc)
            for media_field in media_fields:
                if media_field and isinstance(doc['associations'][media_field], MutableMapping):
                    doc[field_schema] = doc['associations'][media_field]
                    if media_field != 'featuremedia':
                        del doc['associations'][media_field]
                    elif not doc.get('feature_media', None) is None and 'description_text' in doc['feature_media']:
                        doc['media_description'] = doc['feature_media']['description_text']

    def _process_sms(self, doc, schema):
        """Apply the SMS validation to the sms_message value if the document is flagged for SMS
        :param doc:
        :param schema:
        :return:
        """

        if doc.get('flags', {}).get('marked_for_sms', False):
            doc['sms'] = doc.get('sms_message', '')
        else:
            # remove it from the valiadation it is not required
            schema.pop('sms', None) if schema.get('sms') else schema.pop('sms_message', None)

    def _process_media_metadata(self, doc, schema):
        """Request media validation if associations is present

        :param doc:
        :param schema:
        :return:
        """
        if doc.get('associations'):
            schema.setdefault('associations', {})['media_metadata'] = app.config.get(
                'VALIDATE_MEDIA_METADATA_ON_PUBLISH', True)

    def _get_validator_schema(self, validator):
        """Get schema for given validator.

        And make sure there is no `None` value which would raise an exception.
        """
        return {field: get_validator_schema(schema) for field, schema in validator['schema'].items() if schema}

    def _get_vocabulary_display_name(self, vocabulary_id):
        if vocabulary_id == 'anpa_category':
            vocabulary = get_resource_service('vocabularies').find_one(req=None, _id='categories')
        else:
            vocabulary = get_resource_service('vocabularies').find_one(req=None, _id=vocabulary_id)
        if vocabulary and 'display_name' in vocabulary:
            return vocabulary['display_name']
        return vocabulary_id

    def _set_default_subject_scheme(self, item):
        if item.get('subject'):
            for subject in item['subject']:
                subject.setdefault('scheme', None)

    def get_error_field_name(self, error_field):
        if FIELD_LABELS.get(error_field):
            return FIELD_LABELS.get(error_field)
        return error_field

    def _validate(self, doc, fields=False, **kwargs):
        item = deepcopy(doc['validate'])  # make a copy for signal before validation processing
        use_headline = kwargs and 'headline' in kwargs
        validators = self._get_validators(doc)
        for validator in validators:
            validation_schema = self._get_validator_schema(validator)
            self._sanitize_fields(doc['validate'], validator)
            self._set_default_subject_scheme(doc['validate'])
            self._process_media(doc['validate'], validation_schema)
            self._process_sms(doc['validate'], validation_schema)
            self._process_media_metadata(doc['validate'], validation_schema)
            v = SchemaValidator()
            v.allow_unknown = True
            try:
                v.validate(doc['validate'], validation_schema)
            except TypeError as e:
                logger.exception('Invalid validator schema value "%s" for ' % str(e))
            error_list = v.errors
            response = []
            for e in error_list:
                messages = []
                # Ignore dateline if item is corrected because it can't be changed after the item is published
                if doc.get('act', None) == 'correct' and e == 'dateline':
                    continue
                elif doc.get('act', None) == 'kill' and doc['validate'].get('profile', None) and \
                        e in ('headline', 'abstract', 'body_html'):
                    continue
                elif e == 'extra':
                    for field in error_list[e]:
                        display_name = self._get_vocabulary_display_name(field)
                        if 'required' in error_list[e][field]:
                            messages.append(ERROR_MESSAGES[REQUIRED_ERROR].format(display_name))
                        else:
                            error_field = self.get_error_field_name(display_name)
                            messages.append('{} {}'.format(error_field, error_list[e][field]))
                elif 'required field' in error_list[e] or type(error_list[e]) is dict or type(error_list[e]) is list:
                    display_name = self._get_vocabulary_display_name(e)
                    error_field = self.get_error_field_name(display_name)
                    messages.append(ERROR_MESSAGES[REQUIRED_ERROR].format(error_field.upper()))
                elif 'min length is 1' == error_list[e] or 'null value not allowed' in error_list[e]:
                    messages.append(ERROR_MESSAGES[REQUIRED_ERROR].format(e.upper()))
                elif 'min length is' in error_list[e]:
                    error_field = self.get_error_field_name(e)
                    messages.append(ERROR_MESSAGES[TOO_SHORT].format(error_field.upper()))
                elif 'max length is' in error_list[e]:
                    error_field = self.get_error_field_name(e)
                    messages.append(ERROR_MESSAGES[TOO_LONG].format(error_field.upper()))
                else:
                    error_field = self.get_error_field_name(e)
                    messages.append('{} {}'.format(error_field.upper(),
                                    ERROR_MESSAGES[error_list[e]]
                                    if ERROR_MESSAGES.get(error_list[e]) else error_list[e]))

                for message in messages:
                    if use_headline:
                        headline = '{}: {}'.format(doc['validate'].get('headline',
                                                                       doc['validate'].get('_id')), message)
                        response.append(headline)
                    else:
                        response.append(message)

            # let custom code do additional validation
            item_validate.send(self, item=item, response=response, error_fields=v.errors)

            if fields:
                return response, v.errors
            return response
        else:
            logger.warn('validator was not found for {}'.format(doc['act']))
            if fields:
                return [], {}
            return []



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import html
from flask import render_template

from eve.versioning import resolve_document_version
from apps.templates.content_templates import render_content_template_by_name
from .common import BasePublishService, BasePublishResource, ITEM_KILL
from eve.utils import config
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE, PUB_STATUS, EMBARGO, SCHEDULE_SETTINGS, \
    PUBLISH_SCHEDULE
from superdesk import get_resource_service
from superdesk.utc import utcnow
import logging
from copy import deepcopy
from superdesk.emails import send_article_killed_email
from superdesk.errors import SuperdeskApiError
from apps.archive.common import ITEM_OPERATION, ARCHIVE, insert_into_versions, get_dateline_city
from itertools import chain
from apps.publish.published_item import PUBLISHED, LAST_PUBLISHED_VERSION
from flask_babel import _
from enum import Enum
from bson.objectid import ObjectId
from apps.content import push_content_notification

logger = logging.getLogger(__name__)
# what to do when the item is in a package
PACKAGE_WORKFLOW = Enum('PACKAGE_WORKFLOW', (
    # raise an exception, the VALIDATE_ERROR_MESSAGE will be used
    'RAISE',
    # ignore, the action can be done on items in a package
    'IGNORE',
))


class KillPublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, 'kill')


class KillPublishService(BasePublishService):
    publish_type = 'kill'
    published_state = 'killed'
    item_operation = ITEM_KILL
    package_workflow = PACKAGE_WORKFLOW.RAISE
    VALIDATE_ERROR_MESSAGE = _(
        'This item is in a package. It needs to be removed before the item can be killed'
    )

    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource=datasource, backend=backend)

    def on_update(self, updates, original):
        # check if we are trying to kill an item that is contained in package
        # and the package itself is not killed.

        packages = self.package_service.get_packages(original[config.ID_FIELD])
        if self.package_workflow == PACKAGE_WORKFLOW.RAISE:
            if packages and packages.count() > 0:
                for package in packages:
                    if package[ITEM_STATE] not in {
                            CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED, CONTENT_STATE.UNPUBLISHED}:
                        raise SuperdeskApiError.badRequestError(message=self.VALIDATE_ERROR_MESSAGE)
        elif self.package_workflow not in PACKAGE_WORKFLOW:
            raise ValueError("Invalid package workflow")

        updates['pubstatus'] = PUB_STATUS.CANCELED
        updates['versioncreated'] = utcnow()

        super().on_update(updates, original)
        updates[ITEM_OPERATION] = self.item_operation
        self._remove_marked_user(original)
        get_resource_service('archive_broadcast').spike_item(original)

    def update(self, id, updates, original):
        """Kill will broadcast kill email notice to all subscriber in the system and then kill the item.

        Kill for multiple items is triggered
        - for broadcast items if master item is killed.
        In case of the multiple items the kill header text will be different but rest
        of the body_html will be same.

        If any other fields needs in the kill template that needs to be based on the item that is being
        killed (not the item that is being actioned on) then modify the article_killed_override.json file'
        """
        # kill cannot be scheduled and embargoed.
        updates[EMBARGO] = None
        updates[PUBLISH_SCHEDULE] = None
        updates[SCHEDULE_SETTINGS] = {}
        updates_copy = deepcopy(updates)
        original_copy = deepcopy(original)
        self.apply_kill_override(original_copy, updates)
        self.broadcast_kill_email(original, updates)
        super().update(id, updates, original)
        updated = deepcopy(original)
        updated.update(updates)
        get_resource_service('archive_broadcast').kill_broadcast(updates_copy, original_copy, self.item_operation)

    def broadcast_kill_email(self, original, updates):
        """Sends the broadcast email to all subscribers (including in-active subscribers)

        :param dict original: Document to kill
        :param dict updates: kill updates
        """
        # Get all subscribers
        subscribers = list(get_resource_service('subscribers').get(req=None, lookup=None))

        recipients = [s.get('email').split(',') for s in subscribers if s.get('email')]
        recipients = list(set(chain(*recipients)))
        # send kill email.
        kill_article = deepcopy(original)
        kill_article['body_html'] = updates.get('body_html')
        kill_article['headline'] = updates.get('headline')
        kill_article['desk_name'] = get_resource_service('desks').get_desk_name(kill_article.get('task',
                                                                                                 {}).get('desk'))
        kill_article['city'] = get_dateline_city(kill_article.get('dateline'))
        kill_article['action'] = self.item_operation
        send_article_killed_email(kill_article, recipients, utcnow())

    def kill_item(self, updates, original):
        """Kill the item after applying the template.

        :param dict updates:
        :param dict original:
        """
        # apply the kill template
        original_copy = deepcopy(original)

        updates_data = self.apply_kill_template(original_copy)
        updates_data['body_html'] = updates.get('body_html', '')

        # resolve the document version
        resolve_document_version(document=updates_data, resource=ARCHIVE, method='PATCH', latest_doc=original)
        # kill the item
        self.patch(original.get(config.ID_FIELD), updates_data)
        # insert into versions
        insert_into_versions(id_=original[config.ID_FIELD])

    def apply_kill_template(self, item):
        # apply the kill template
        updates = render_content_template_by_name(item, self.item_operation)
        return updates

    def apply_kill_override(self, item, updates):
        """Applies kill override.

        Kill requires content to be generate based on the item getting killed (and not the
        item that is being actioned on).

        :param dict item: item to kill
        :param dict updates: updates that needs to be modified based on the template
        :return:
        """
        try:
            if item.get('_type') == 'archive':
                # attempt to find the published item as this will have an accurate time of publication
                published_item = get_resource_service(PUBLISHED).get_last_published_version(item.get(config.ID_FIELD))
                versioncreated = published_item.get('versioncreated') if published_item else \
                    item.get('versioncreated', item.get(config.LAST_UPDATED))
            else:
                versioncreated = item.get('versioncreated', item.get(config.LAST_UPDATED))
            desk_name = get_resource_service('desks').get_desk_name(item.get('task', {}).get('desk'))
            city = get_dateline_city(item.get('dateline'))
            kill_header = json.loads(render_template('article_killed_override.json',
                                                     slugline=item.get('slugline', ''),
                                                     headline=item.get('headline', ''),
                                                     desk_name=desk_name,
                                                     city=city,
                                                     versioncreated=versioncreated,
                                                     body_html=updates.get('body_html', ''),
                                                     update_headline=updates.get('headline', ''),
                                                     item_operation=self.item_operation.lower()),
                                     strict=False)

            for key, value in kill_header.items():
                kill_header[key] = html.unescape(value)

            updates.update(kill_header)
        except Exception:
            logger.exception('Failed to apply kill header template to item {}.'.format(item))

    def _remove_marked_user(self, item):
        """Remove the marked_for_user from all the published items having same 'item_id' as item being killed."""
        item_id = item.get('_id')
        if not item_id:
            return

        updates = {'marked_for_user': None}
        published_service = get_resource_service(PUBLISHED)

        published_items = list(published_service.get_from_mongo(req=None, lookup={'item_id': item_id}))
        if not published_items:
            return

        for item in published_items:
            if item and item.get('marked_for_user'):
                updated = item.copy()
                updated.update(updates)

                published_service.system_update(
                    ObjectId(item.get('_id')), updates, item
                )
                # send notifications so that list can be updated in the client
                get_resource_service('archive').handle_mark_user_notifications(updates, item, False)
                push_content_notification([updated, item])



import os
from unittest import mock

from superdesk import get_resource_service
from superdesk.io.feeding_services import FTPFeedingService
from superdesk.io.commands.update_ingest import ingest_items

from .mocks_reuters import setup_reuters_mock, teardown_reuters_mock


def setup_providers(context):
    app = context.app
    context.providers = {}
    context.ingest_items = ingest_items
    with app.test_request_context(app.config['URL_PREFIX']):
        rule_sets = {'name': 'reuters_rule_sets',
                     'rules': [
                         {"old": "@", "new": ""},
                     ]}

        result = get_resource_service('rule_sets').post([rule_sets])

        app.config['REUTERS_USERNAME'] = 'no_username'
        app.config['REUTERS_PASSWORD'] = 'no_password'
        setup_reuters_mock(context)

        path_to_fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), '../../tests/io/fixtures')
        providers = [
            {
                'name': 'reuters',
                'source': 'reuters',
                'feeding_service': 'reuters_http',
                'feed_parser': 'newsml2',
                'is_closed': False,
                'rule_set': result[0],
                'config': {
                    'url': 'http://rmb.reuters.com/rmd/rest/xml',
                    'auth_url': 'https://commerce.reuters.com/rmd/rest/xml/login',
                    'username': app.config['REUTERS_USERNAME'], 'password': app.config['REUTERS_PASSWORD']
                }
            },
            {
                'name': 'AAP',
                'source': 'AAP Ingest',
                'feeding_service': 'file',
                'feed_parser': 'nitf',
                'is_closed': False,
                'critical_errors': {'2005': True},
                'config': {'path': path_to_fixtures}
            },
            {
                'name': 'teletype',
                'source': 'AAP Teletype',
                'feeding_service': 'file',
                'feed_parser': 'zczc',
                'is_closed': False,
                'config': {'path': path_to_fixtures}
            },
            {
                'name': 'DPA',
                'source': 'DPA',
                'feeding_service': 'file',
                'feed_parser': 'dpa_iptc7901',
                'is_closed': False,
                'config': {'path': path_to_fixtures}
            },
            {
                'name': 'AP',
                'source': 'AP',
                'feeding_service': 'file',
                'feed_parser': 'ap_anpa1312',
                'is_closed': False,
                'config': {'path': path_to_fixtures}
            },
            {
                'name': 'STT',
                'source': 'STT',
                'feeding_service': 'file',
                'feed_parser': 'sttnewsml',
                'is_closed': False,
                'config': {'path': path_to_fixtures}
            },
            {
                'name': 'ninjs',
                'source': 'NINJS',
                'feeding_service': 'file', 'feed_parser': 'ninjs',
                'is_closed': False,
                'config': {'path': path_to_fixtures}
            },
            {
                'name': 'email',
                'source': 'Email',
                'feeding_service': 'file',
                'feed_parser': 'email_rfc822',
                'is_closed': False,
                'config': {'path': path_to_fixtures, 'formatted': True}
            },
            {
                'name': 'ftp_ninjs',
                'source': 'ftp ninjs',
                'feeding_service': 'ftp',
                'feed_parser': 'ninjs',
                'is_closed': False,
                'config': {
                    'path_fixtures': path_to_fixtures,
                    'formatted': True,
                    'host': 'fake-ftp.example.com',
                    'username': 'admin',
                    'password': 'admin',
                    'path': '/admin/superdesk',
                    'dest_path': '/tmp'
                }
            }
        ]

        with mock.patch.object(FTPFeedingService, '_test', return_value=True):
            result = get_resource_service('ingest_providers').post(providers)

        context.providers['reuters'] = result[0]
        context.providers['aap'] = result[1]
        context.providers['teletype'] = result[2]
        context.providers['dpa'] = result[3]
        context.providers['ap'] = result[4]
        context.providers['ninjs'] = result[5]
        context.providers['email'] = result[6]
        context.providers['ftp_ninjs'] = result[7]


def teardown_providers(context):
    teardown_reuters_mock(context)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from bson import ObjectId
from unittest import mock
from unittest.mock import MagicMock
from datetime import timedelta

from superdesk.tests import TestCase
from superdesk.utc import utcnow
from apps.publish.enqueue import enqueue_service
from superdesk.publish.publish_queue import PUBLISHED_IN_PACKAGE
from superdesk.publish import publish_queue
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE


class QueueItemsTestCase(TestCase):
    """Tests for the get_queue_items() function."""

    def setUp(self):
        try:
            from superdesk.publish.publish_content import get_queue_items
        except ImportError:
            self.fail("Could not import function under test (transmit_items).")
        else:
            self.func_under_test = get_queue_items
            self.queue_items = [
                {'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_1', 'item_version': 4,
                 'headline': 'pending headline', 'destination': {}},
                {'_id': ObjectId(), 'state': 'retrying', 'item_id': 'item_2', 'item_version': 4,
                 'headline': 'retrying headline', 'retry_attempt': 2,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=30)},
                {'_id': ObjectId(), 'state': 'success', 'item_id': 'item_3', 'item_version': 4,
                 'headline': 'success headline', 'retry_attempt': 4,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'failed', 'item_id': 'item_4', 'item_version': 4,
                 'headline': 'failed headline', 'retry_attempt': 10,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'canceled', 'item_id': 'item_5', 'item_version': 4,
                 'headline': 'canceled headline', 'retry_attempt': 4,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'retrying', 'item_id': 'item_6', 'item_version': 4,
                 'headline': 'retrying headline', 'retry_attempt': 2,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
            ]
            self.app.data.insert('publish_queue', self.queue_items)

    def test_get_queue_items(self):
        items = list(self.func_under_test())
        self.assertEqual(len(items), 1)
        for item in items:
            self.assertIn(item['item_id'], ['item_1'])

    def test_get_retry_queue_items(self):
        items = list(self.func_under_test(True))
        self.assertEqual(len(items), 1)
        for item in items:
            self.assertIn(item['item_id'], ['item_6'])

    def test_get_queue_items_with_retrying_items(self):
        item = self.app.data.find_one('publish_queue', req=None, _id=self.queue_items[1]['_id'])
        self.app.data.update('publish_queue', item.get('_id'),
                             {'next_retry_attempt_at': utcnow() - timedelta(minutes=30)},
                             item)
        items = list(self.func_under_test(True))
        self.assertEqual(len(items), 2)
        self.assertListEqual([item_l['item_id'] for item_l in items], ['item_2', 'item_6'])

    @mock.patch.object(enqueue_service, 'ObjectId')
    @mock.patch.object(enqueue_service, 'get_utc_schedule')
    @mock.patch.object(enqueue_service, 'get_resource_service')
    @mock.patch.object(enqueue_service, 'get_formatter')
    def test_enqueue_dict(self, *mocks):
        get_formatter, get_resource_service, _, _ = mocks
        publish_queue = get_resource_service.return_value
        fake_post = publish_queue.post
        service = enqueue_service.EnqueueService()
        fake_formatter = get_formatter.return_value
        doc_dict = {ITEM_TYPE: CONTENT_TYPE.TEXT, PUBLISHED_IN_PACKAGE: False}
        fake_doc = MagicMock()
        fake_doc.__getitem__ = lambda s, k: doc_dict.get(k, MagicMock())
        fake_doc.get = doc_dict.get
        fake_destination = MagicMock()
        fake_subscriber = MagicMock()
        subs_dict = {'destinations': [fake_destination], 'api_enabled': False}
        fake_subscriber.__getitem__ = lambda s, k: subs_dict.get(k, MagicMock())
        fake_subscriber['destinations'] = [fake_destination]
        subscribers = [fake_subscriber]
        fake_formatter.format.return_value = [{'published_seq_num': 42,
                                               'formatted_item': 'test OK'}]
        service.get_destinations = MagicMock(return_value=fake_subscriber['destinations'])
        service.queue_transmission(fake_doc, subscribers)
        self.assertEqual(len(fake_post.call_args_list), 1)
        self.assertEqual(len(fake_post.call_args_list[0][0][0]), 1)
        doc = fake_post.call_args_list[0][0][0][0]
        self.assertEqual(doc['published_seq_num'], 42)
        self.assertEqual(doc['formatted_item'], 'test OK')

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [{'this_should_not': 'work',
                                               'bad_key': 'value'}]
        service.queue_transmission(fake_doc, subscribers)
        # post should not have been called here,
        # because the dict is lacking the mandatory keys
        self.assertFalse(fake_post.called)

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [(42, 'test tuple OK')]
        service.queue_transmission(fake_doc, subscribers)
        self.assertEqual(len(fake_post.call_args_list), 1)
        self.assertEqual(len(fake_post.call_args_list[0][0][0]), 1)
        doc = fake_post.call_args_list[0][0][0][0]
        self.assertEqual(doc['published_seq_num'], 42)
        self.assertEqual(doc['formatted_item'], 'test tuple OK')

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [(1, "2", 3)]
        service.queue_transmission(fake_doc, subscribers)
        # post should not have been called here,
        # because the tuple should be in (published_seq_num, formatted_item) format
        self.assertFalse(fake_post.called)

    @mock.patch.object(publish_queue, 'app')
    def test_delete_encoded_item(self, fake_app):
        fake_storage = fake_app.storage
        fake_storage_delete = fake_storage.delete
        service = publish_queue.PublishQueueService(backend=MagicMock())
        service.get_from_mongo = MagicMock()
        service.get_from_mongo.return_value = [{'_id': "4567", 'encoded_item_id': 'TEST ID'}]
        service.delete({'_id': "4567"})
        assert fake_storage_delete.call_args == mock.call('TEST ID')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .area_square_feet_to_metric import square_feet_to_metric


class AreaTestCase(unittest.TestCase):

    def test_f_to_c(self):
        text = 'Total area is 100.50 square feet for this land. '\
               'Total area is 15.7 square ft for this land. '\
               'Total area is 1 Square Foot for this land. '\
               'Total area is 1-16 sq-ft for this land. '\
               'Total area is 16.7-Square-ft for this land. '\
               'Total area is 16,500-sq. ft for this land. '\

        item = {'body_html': text}
        res, diff = square_feet_to_metric(item)
        self.assertEqual(diff['100.50 square feet'], '100.50 square feet (9.3 square meter)')
        self.assertEqual(diff['15.7 square ft'], '15.7 square ft (1.5 square meter)')
        self.assertEqual(diff['1 Square Foot'], '1 Square Foot (0.1 square meter)')
        self.assertEqual(diff['1-16 sq-ft'], '1-16 sq-ft (0.1-1.5 square meter)')
        self.assertEqual(diff['16.7-Square-ft'], '16.7-Square-ft (1.6 square meter)')
        self.assertEqual(diff['16,500-sq. ft'], '16,500-sq. ft (1,533 square meter)')
        self.assertEqual(res['body_html'], item['body_html'])
